@@ -705,10 +705,13 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 			break;
 		    } else {
 			snprintf(url, 1024, "%s", p);
-			sub = strchr(url, 0xd);
+			sub = strchr(url, 0xd);	   // terminate on CR
 			if (sub != NULL)
 			    sub[0] = '\0';
-			sub = strchr(url, 0xa);
+			sub = strchr(url, 0xa);	   // terminate on line feed
+			if (sub != NULL)
+			    sub[0] = '\0';
+			sub = strchr(url, 0x20);   // terminate on space
 			if (sub != NULL)
 			    sub[0] = '\0';
 			if (DEBUG)
@@ -737,6 +740,9 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 			if (sub != NULL)
 			    sub[0] = '\0';
 			sub = strchr(url, 0xa);
+			if (sub != NULL)
+			    sub[0] = '\0';
+			sub = strchr(url, 0x20);   // terminate on space
 			if (sub != NULL)
 			    sub[0] = '\0';
 			if (DEBUG)
@@ -762,6 +768,9 @@ void buildPlaylist(nsPluginInstance * instance, char *file, Node * parent)
 			if (sub != NULL)
 			    sub[0] = '\0';
 			sub = strchr(url, 0xa);
+			if (sub != NULL)
+			    sub[0] = '\0';
+			sub = strchr(url, 0x20);   // terminate on space
 			if (sub != NULL)
 			    sub[0] = '\0';
 			if (DEBUG)
