@@ -4,6 +4,8 @@
 #ifndef STATICDECLS
 #define STATICDECLS
 int DEBUG = 0;
+int instance_counter = 0;
+
 #define MAX_BUF_LEN 255
 #define STATE_RESET 0
 #define STATE_NEW 1
@@ -50,6 +52,8 @@ char *GetMIMEDescription()
     int enable_rm;
     int enable_gmp;
     int enable_mp3;
+    int enable_midi;
+    int enable_pls;
 
 #ifdef STD
     FILE *customtypes;
@@ -78,6 +82,8 @@ char *GetMIMEDescription()
     enable_qt = 1;
     enable_rm = 1;
     enable_gmp = 1;
+    enable_midi = 0;
+    enable_pls = 0;
     use_custom_mime_types = 0;
 
     DEBUG = 0;			//set to 1 when debugging this function
@@ -192,6 +198,24 @@ char *GetMIMEDescription()
 			printf("mp3:%i\n", enable_mp3);
 		    continue;
 		}
+		// MIDI
+		if (strncasecmp(buffer, "enable-midi", 11) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &enable_midi);
+		    if (DEBUG)
+			printf("midi:%i\n", enable_midi);
+		    continue;
+		}
+		// PLS
+		if (strncasecmp(buffer, "enable-pls", 10) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &enable_pls);
+		    if (DEBUG)
+			printf("pls:%i\n", enable_pls);
+		    continue;
+		}
 
 	    }
 	    fclose(config);
@@ -285,7 +309,7 @@ char *GetMIMEDescription()
 		    "audio/x-pn-realaudio:ram,rm:RealAudio;"
 		    "application/vnd.rn-realmedia:rm:RealMedia;"
 		    "application/vnd.rn-realaudio:ra,ram:RealAudio;"
-		    "video/vnd.rn-realvideo:rv:RealVideo;"	
+		    "video/vnd.rn-realvideo:rv:RealVideo;"
 		    "audio/x-realaudio:ra:RealAudio;", sizeof(MimeTypes));
 
 	    if (enable_helix)
@@ -316,6 +340,7 @@ char *GetMIMEDescription()
 		strlcat(MimeTypes,
 			"audio/mpeg3:mp3:MPEG audio;"
 			"audio/x-mpeg3:mp3:MPEG audio;"
+			"audio/x-mpegurl:m3u:MPEG url;"
 			"audio/mp3:mp3:MPEG audio;", sizeof(MimeTypes));
 
 	}
@@ -342,12 +367,29 @@ char *GetMIMEDescription()
 		sizeof(MimeTypes));
 	// DIVX
 	strlcat(MimeTypes,
+		"video/divx:divx:DivX Media Format;"
 		"video/vnd.divx:divx:DivX Media Format;",
 		sizeof(MimeTypes));
 	// Basic
 	strlcat(MimeTypes,
 		"audio/basic:au,snd:Basic Audio File;"
-    		"audio/x-basic:au,snd:Basic Audio File;", sizeof(MimeTypes));
+		"audio/x-basic:au,snd:Basic Audio File;",
+		sizeof(MimeTypes));
+
+	// MIDI
+	if (enable_midi) {
+	    strlcat(MimeTypes,
+		    "audio/midi:mid,midi,kar:MIDI Audio;",
+		    sizeof(MimeTypes));
+	}
+
+	// PLS
+	if (enable_pls) {
+	    strlcat(MimeTypes,
+   		    "audio/x-scpls:pls:Shoutcast Playlist;",
+		    sizeof(MimeTypes));
+	}
+
 #endif
 	// Google Video
 #ifdef GMP
@@ -470,6 +512,10 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		printf("Argument Value: %s\n", parameters->argv[i]);
 	    }
 
+	    if (parameters->argn[i] == NULL || parameters->argv[i] == NULL) {
+		break;
+	    }
+
 	    if (strncasecmp(parameters->argn[i], "debug", 5) == 0) {
 		lowercase(parameters->argv[i]);
 
@@ -548,11 +594,15 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 	    }
 
 	    if (strncasecmp(parameters->argn[i], "height", 6) == 0) {
-		sscanf(parameters->argv[i], "%i", &instance->embed_height);
+		if (strstr(parameters->argv[i], "%") == NULL)
+		    sscanf(parameters->argv[i], "%i",
+			   &instance->embed_height);
 	    }
 
 	    if (strncasecmp(parameters->argn[i], "width", 5) == 0) {
-		sscanf(parameters->argv[i], "%i", &instance->embed_width);
+		if (strstr(parameters->argv[i], "%") == NULL)
+		    sscanf(parameters->argv[i], "%i",
+			   &instance->embed_width);
 	    }
 
 	    if (strncasecmp(parameters->argn[i], "starttime", 9) == 0) {
@@ -760,11 +810,7 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		if (DEBUG)
 		    printf("argv[i]=%s\n", parameters->argv[i]);
 
-		if (strstr(parameters->argv[i], "true")
-		    || strstr(parameters->argv[i], "yes")
-		    || strstr(parameters->argv[i], "1")
-		    || strstr(parameters->argv[i], "all")
-		    || strstr(parameters->argv[i], "statusfield")
+		if (strstr(parameters->argv[i], "statusfield")
 		    || strstr(parameters->argv[i], "statuspanel")
 		    || strstr(parameters->argv[i], "playbutton")
 		    || strstr(parameters->argv[i], "volumeslider")
@@ -774,11 +820,17 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		} else {
 		    instance->controlwindow = 0;
 		}
-	        if (strstr(parameters->argv[i], "controlpanel") 
+		if (strstr(parameters->argv[i], "controlpanel")
 		    || strstr(parameters->argv[i], "statusbar")) {
-			instance->showcontrols = 1;
+		    instance->showcontrols = 1;
+		    if (instance_counter > 1)
+			instance->controlwindow = 1;
 		}
-		if (strstr(parameters->argv[i], "imagewindow")) {
+		if (strstr(parameters->argv[i], "imagewindow")
+		    || strstr(parameters->argv[i], "true")
+		    || strstr(parameters->argv[i], "yes")
+		    || strstr(parameters->argv[i], "1")
+		    || strstr(parameters->argv[i], "all")) {
 		    instance->controlwindow = 0;
 		}
 		if (DEBUG)
@@ -791,15 +843,15 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 		lowercase(parameters->argv[i]);
 		if (DEBUG)
 		    printf("argv[i]=%s\n", parameters->argv[i]);
-		
+
 		if (strstr(parameters->argv[i], "_master")) {
-			real_master_console = 1;
+		    real_master_console = 1;
 		} else {
-			real_master_console = 0;
+		    real_master_console = 0;
 		}
 
-	    }		
-	
+	    }
+
 	    if ((strncasecmp
 		 (parameters->argn[i], "onmediacomplete", 15) == 0)
 		|| (strncasecmp(parameters->argn[i], "onendofstream", 13)
@@ -1034,7 +1086,7 @@ void New(nsPluginInstance * instance, nsPluginCreateData * parameters)
 	}
 	if (instance->controlwindow == 1 && real_master_console == 1)
 	    instance->controlwindow = 0;
-    
+
 	if (instance->controlwindow == 0) {
 	    if ((instance->fname != NULL)
 		&& (!isMms(instance->fname, instance->nomediacache))) {
@@ -1127,7 +1179,39 @@ void LoadConfigFile(nsPluginInstance * instance)
 		    continue;
 		}
 
-		if (strncasecmp(buffer, "novop", 5) == 0) {
+	        if (strncasecmp(buffer, "showstatus", 10) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    // crappy reuse of the hidestatus variable
+		    sscanf(parse, "%i", &instance->hidestatus);
+		    // flip the logic here
+		    if (instance->hidestatus == 0) {
+			instance->hidestatus = 1;
+		    } else {
+		    	instance->hidestatus = 0;
+		    }
+		    continue;
+		}
+
+		if (strncasecmp(buffer, "showtracker", 11) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->showtracker);
+		    if (instance->showtracker != 0)
+			instance->showtracker = 1;
+		    continue;
+		}
+
+		if (strncasecmp(buffer, "showcontrols", 12) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->showcontrols);
+		    if (instance->showcontrols != 0)
+			instance->showcontrols = 1;
+		    continue;
+		}
+
+	        if (strncasecmp(buffer, "novop", 5) == 0) {
 		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
 		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
 		    sscanf(parse, "%i", &instance->novop);
@@ -1192,6 +1276,15 @@ void LoadConfigFile(nsPluginInstance * instance)
 		    continue;
 		}
 
+		if (strncasecmp(buffer, "rtsp-use-http", 13) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->rtsp_use_http);
+		    if (instance->rtsp_use_http != 0)
+			instance->rtsp_use_http = 1;
+		    continue;
+		}
+
 		if (strncasecmp(buffer, "qt-speed", 8) == 0) {
 		    sprintf(parse, "%s", strtok(buffer, "="));
 		    sprintf(parse, "%s", strtok(NULL, "="));
@@ -1436,6 +1529,24 @@ void LoadConfigFile(nsPluginInstance * instance)
 			printf("mp3:%i\n", instance->enable_mp3);
 		    continue;
 		}
+		// MIDI
+		if (strncasecmp(buffer, "enable-midi", 11) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->enable_midi);
+		    if (DEBUG)
+			printf("midi:%i\n", instance->enable_midi);
+		    continue;
+		}
+		// PLS
+		if (strncasecmp(buffer, "enable-pls", 10) == 0) {
+		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
+		    snprintf(parse, 1000, "%s", strtok(NULL, "="));
+		    sscanf(parse, "%i", &instance->enable_pls);
+		    if (DEBUG)
+			printf("pls:%i\n", instance->enable_pls);
+		    continue;
+		}
 		// Ogg
 		if (strncasecmp(buffer, "enable-ogg", 10) == 0) {
 		    snprintf(parse, 1000, "%s", strtok(buffer, "="));
