@@ -196,13 +196,15 @@ void mmsToHttp(char *dest, char *src)
 {
     char *temp;
 
-    temp = (char *) NPN_MemAlloc(sizeof(char) * (strlen(src) + 1 +
-						 sizeof("http") -
-						 sizeof("mms")));
-    strcpy(temp, "http");
-    strcat(temp, src + 3);
-    strcpy(dest, temp);
-    NPN_MemFree(temp);
+    if (strncasecmp(src,"mms",3) == 0) {
+	temp = (char *) NPN_MemAlloc(sizeof(char) * (strlen(src) + 1 +
+						     sizeof("http") -
+						     sizeof("mms")));
+	strcpy(temp, "http");
+	strcat(temp, src + 3);
+	strcpy(dest, temp);
+	NPN_MemFree(temp);
+    }
     return;
 }
 
@@ -465,7 +467,8 @@ int sendCommand(nsPluginInstance * instance, char *command)
 	//operations on pipes are atomic
 
 	if (retval < (int) strlen(buffer)) {
-	    printf("*****sendCommand Truncated*****\n");
+		if (DEBUG)
+	    		printf("*****sendCommand Truncated*****\n");
 
 	}
 
@@ -581,9 +584,9 @@ void killmplayer(nsPluginInstance * instance)
 	}
 	instance->threadsetup = 0;
 #ifdef GTK_ENABLED
-    	if (GTK_IS_WIDGET(instance->drawing_area)) {
-		gtk_widget_destroy(instance->drawing_area);
-		instance->drawing_area = NULL;
+	if (GTK_IS_WIDGET(instance->drawing_area)) {
+	    gtk_widget_destroy(instance->drawing_area);
+	    instance->drawing_area = NULL;
 	}
 #endif
     }
