@@ -151,7 +151,8 @@ void signalPlayerThread(nsPluginInstance * instance)
 	pthread_suspend(10);
 	pthread_mutex_lock(&(instance->control_mutex));
     }
-    if (instance->js_state == JS_STATE_BUFFERING) {
+    if (instance->js_state == JS_STATE_BUFFERING
+	|| instance->js_state == JS_STATE_READY) {
 	pthread_mutex_lock(&(instance->playlist_cond_mutex));
 	pthread_cond_signal(&(instance->playlist_complete_cond));
 	pthread_mutex_unlock(&(instance->playlist_cond_mutex));
@@ -259,11 +260,11 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	printf("Entering SetupPlayer\n");
 
 #ifdef X_ENABLED
-    DrawUI(instance->widget, instance, "Loading Media...", 0, -1);
+    DrawUI(instance->widget, instance, _("Loading Media..."), 0, -1);
 #endif
 #ifdef GTK_ENABLED
     if (instance->status != NULL)
-	gtk_label_set_text(instance->status, "Loading Media...");
+	gtk_label_set_text(instance->status, _("Loading Media..."));
 #endif
 
     if (instance->td->list == NULL) {
@@ -393,25 +394,28 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	    instance->td->argv[i++] = strdup(buffer);
 #endif
 #ifdef GTK_ENABLED
-	    instance->drawing_area = gtk_drawing_area_new();
-	    gtk_widget_set_events(instance->drawing_area,
-				  GDK_BUTTON_PRESS_MASK);
-	    gtk_fixed_put(GTK_FIXED(instance->fixed_container),
-			  instance->drawing_area, 0, 0);
-	    gtk_widget_show(instance->drawing_area);
 	    if (instance->targetplayer == 1)
 		gtk_widget_show(instance->gtkwidget);
+#ifdef GTK2_ENABLED
+	    instance->player_window =
+		gtk_socket_get_id(GTK_SOCKET(instance->drawing_area));
+#endif
+#ifdef GTK1_ENABLED
 	    instance->player_window =
 		GDK_WINDOW_XWINDOW(instance->drawing_area->window);
-	    if (instance->targetplayer == 1)
-		gtk_widget_hide(instance->gtkwidget);
+#endif				/* if (instance->targetplayer == 1) */
 	    snprintf(buffer, 1024, "0x%x", (int) instance->player_window);
 	    instance->td->argv[i++] = strdup(buffer);
 #ifdef GTK2_ENABLED
-	    g_signal_connect_after(G_OBJECT(instance->gtkwidget),
-				   "visibility-notify-event",
-				   G_CALLBACK(window_visible), instance);
+	    instance->visible_signal_id =
+		g_signal_connect_after(G_OBJECT(instance->gtkwidget),
+				       "visibility-notify-event",
+				       G_CALLBACK(window_visible),
+				       instance);
 #endif
+	    if (instance->targetplayer == 1)
+		gtk_widget_hide(instance->gtkwidget);
+	    
 #endif
 	} else {
 	    instance->player_window = 0;
@@ -427,16 +431,16 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	    instance->td->argv[i++] = strdup(buffer);
 #endif
 #ifdef GTK_ENABLED
-	    instance->drawing_area = gtk_drawing_area_new();
-	    gtk_widget_set_events(instance->drawing_area,
-				  GDK_BUTTON_PRESS_MASK);
-	    gtk_fixed_put(GTK_FIXED(instance->fixed_container),
-			  instance->drawing_area, 10, 100);
 	    gtk_widget_set_usize(GTK_WIDGET(instance->status),
 				 instance->window_width - 20, 19);
-	    gtk_widget_show(instance->drawing_area);
+#ifdef GTK2_ENABLED
+	    instance->player_window =
+		gtk_socket_get_id(GTK_SOCKET(instance->drawing_area));
+#endif
+#ifdef GTK1_ENABLED
 	    instance->player_window =
 		GDK_WINDOW_XWINDOW(instance->drawing_area->window);
+#endif
 	    snprintf(buffer, 1024, "0x%x", (int) instance->player_window);
 	    instance->td->argv[i++] = strdup(buffer);
 #endif
@@ -469,9 +473,9 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	    if ((isRemoteDisplay == false)
 		&& (instance->targetplayer == 0)) {
 		if (instance->maintain_aspect == 1) {
-		    snprintf(buffer, 1024, "-xy");
+		    snprintf(buffer, 1024, "-vf");
 		    instance->td->argv[i++] = strdup(buffer);
-		    snprintf(buffer, 1024, "%s", xval);
+		    snprintf(buffer, 1024, "scale=%s:-3", xval);
 		    instance->td->argv[i++] = strdup(buffer);
 		} else {
 		    snprintf(buffer, 1024, "-x");
@@ -558,13 +562,6 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
 	instance->td->argv[i++] = strdup(buffer);
     }
 
-    if (instance->useragent) {
-	snprintf(buffer, 1024, "-user-agent");
-	instance->td->argv[i++] = strdup(buffer);
-	snprintf(buffer, 1024, "%s", instance->useragent);
-	instance->td->argv[i++] = strdup(buffer);
-    }
-
     snprintf(buffer, 1024, "-osdlevel");
     instance->td->argv[i++] = strdup(buffer);
     snprintf(buffer, 1024, "%i", instance->osdlevel);
@@ -598,11 +595,11 @@ void SetupPlayer(nsPluginInstance * instance, XEvent * event)
     if (DEBUG)
 	printf("creating thread - NP_EMBED\n");
 #ifdef X_ENABLED
-    DrawUI(instance->widget, instance, "Getting playlist...", 0, -1);
+    DrawUI(instance->widget, instance, _("Getting playlist..."), 0, -1);
 #endif
 #ifdef GTK_ENABLED
     if (instance->status != NULL)
-	gtk_label_set_text(instance->status, "Getting playlist...");
+	gtk_label_set_text(instance->status, _("Getting playlist..."));
 #endif
 
     if (instance->state < STATE_GETTING_PLAYLIST)
@@ -745,14 +742,14 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
     pthread_mutex_unlock(&(local_td->instance->control_mutex));
     pthread_cleanup_pop(0);
 #ifdef GTK_ENABLED
-	g_idle_add(gtkgui_save_enable, local_td->instance);
-//	g_idle_add(gtkgui_refreshbuttonstate, local_td->instance);
+    g_idle_add(gtkgui_save_enable, local_td->instance);
+//      g_idle_add(gtkgui_refreshbuttonstate, local_td->instance);
 #endif
 
     // set this to 0 for every new media file;
     local_td->instance->movie_height = 0;
     local_td->instance->movie_width = 0;
-    
+
     while (1) {
 	pthread_testcancel();
 	if (local_td->instance->player == NULL)
@@ -771,8 +768,8 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 
 	pthread_testcancel();
 #ifdef GTK_ENABLED
-//	g_idle_add(gtkgui_save_enable, local_td->instance);
-//	g_idle_add(gtkgui_refreshbuttonstate, local_td->instance);
+//      g_idle_add(gtkgui_save_enable, local_td->instance);
+//      g_idle_add(gtkgui_refreshbuttonstate, local_td->instance);
 #endif
 
 	pthread_testcancel();
@@ -789,18 +786,19 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 	    i = 0;
 	    do {
 		pthread_testcancel();
-	    	// need to lock around the read
+		// need to lock around the read
 		pthread_mutex_lock(&(local_td->instance->read_mutex));
 
-	    	if ((local_td->instance->cancelled == 0) && (local_td->instance->player != NULL)) {
-			// c = fgetc(local_td->instance->player);
-			fread(&c,1,1,local_td->instance->player);
+		if ((local_td->instance->cancelled == 0)
+		    && (local_td->instance->player != NULL)) {
+		    // c = fgetc(local_td->instance->player);
+		    fread(&c, 1, 1, local_td->instance->player);
 		} else {
-			c = EOF;
+		    c = EOF;
 		}
 		pthread_mutex_unlock(&(local_td->instance->read_mutex));
 
-	    	if (c == EOF) {
+		if (c == EOF) {
 		    buffer[i] = '\0';
 		    break;
 		}
@@ -849,7 +847,7 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 	    // Only update the display if there is something worth displaying
 	    if (strstr(eos, "Cache fill:") != NULL) {
 		cf = strstr(eos, "Cache fill:");
-	    	lastpercent = (int)cfpercent;
+		lastpercent = (int) cfpercent;
 		i = sscanf(cf, "Cache fill: %lf %% (%ld bytes)",
 			   &cfpercent, &cfbytes);
 		//the following is a workaround for an mplayer bug
@@ -874,8 +872,7 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 		    break;
 		}
 
-		strlcpy(message, "Buffering ", 1024);
-		strlcat(message, local_url, 1000);
+		snprintf(message, 1024, _("Buffering %s"), local_url);
 
 #ifdef X_ENABLED
 		DrawUI(local_td->instance->widget,
@@ -885,23 +882,22 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 		snprintf(local_td->instance->lastmessage, 1024, "%s",
 			 message);
 
-	    	local_td->instance->percent = (cfpercent / 100.0);
+		local_td->instance->percent = (cfpercent / 100.0);
 		if (lastpercent != cfpercent) {
-			g_idle_add(gtkgui_message, local_td->instance);
-			g_idle_add(gtkgui_progress, local_td->instance);
+		    g_idle_add(gtkgui_message, local_td->instance);
+		    g_idle_add(gtkgui_progress, local_td->instance);
 		}
 #endif
-	    pthread_mutex_lock(&(local_td->instance->control_mutex));
-	    local_td->instance->js_state = JS_STATE_BUFFERING;
-	    pthread_mutex_unlock(&(local_td->instance->control_mutex));
-	    
+		pthread_mutex_lock(&(local_td->instance->control_mutex));
+		local_td->instance->js_state = JS_STATE_BUFFERING;
+		pthread_mutex_unlock(&(local_td->instance->control_mutex));
+
 	    }
 	}
 
 	pthread_testcancel();
 	if (strstr(buffer, "Starting") != NULL) {
-	    strlcpy(message, "Playing ", 1024);
-	    strlcat(message, local_url, 1000);
+	    snprintf(message, 1024, _("Playing %s"), local_url);
 #ifdef X_ENABLED
 	    DrawUI(local_td->instance->widget,
 		   local_td->instance, message, 0, -1);
@@ -932,7 +928,9 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 			local_td->instance->embed_height -
 			local_list->play_y;
 
-		if (local_td->instance->player_window != 0 && local_td->instance->movie_height != local_list->play_y) { 
+		if (local_td->instance->player_window != 0
+		    && local_td->instance->movie_height !=
+		    local_list->play_y) {
 
 		    local_td->instance->movie_height = local_list->play_y;
 		    local_td->instance->movie_width = local_list->play_x;
@@ -949,7 +947,9 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 
 	    } else {
 		local_td->instance->panel_height = 16;
-		if (local_td->instance->player_window != 0 && local_td->instance->movie_height != local_list->play_y) {
+		if (local_td->instance->player_window != 0
+		    && local_td->instance->movie_height !=
+		    local_list->play_y) {
 		    local_td->instance->movie_height = local_list->play_y;
 		    local_td->instance->movie_width = local_list->play_x;
 
@@ -975,8 +975,8 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 	    if (local_td->instance->panel_height == 0)
 		local_td->instance->panel_height = 32;
 	    local_td->instance->noredraw = 0;
-	    local_td->instance->movie_height = local_list->play_y - 32;
-	    local_td->instance->movie_width = local_list->play_x;
+	    local_td->instance->movie_height = 1;
+	    local_td->instance->movie_width = 1;
 #ifdef GTK_ENABLED
 	    g_idle_add(gtkgui_resize, local_td->instance);
 	    g_idle_add(gtkgui_draw, local_td->instance);
@@ -1006,14 +1006,16 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 		printf("Media Position = %f\n",
 		       local_td->instance->mediaPos);
 	    if ((int) local_td->instance->mediaLength > 1) {
-	    	lastmediapercent = local_td->instance->mediaPercent;
+		lastmediapercent = local_td->instance->mediaPercent;
 		local_td->instance->mediaPercent =
 		    (int) ((local_td->instance->mediaPos * 100) /
 			   local_td->instance->mediaLength);
 #ifdef GTK_ENABLED
-	    	if (lastmediapercent != local_td->instance->mediaPercent) {
-		    g_idle_add(gtkgui_drawMediaProgress, local_td->instance);
-		    g_idle_add(gtkgui_refreshbuttonstate, local_td->instance);
+		if (lastmediapercent != local_td->instance->mediaPercent) {
+		    g_idle_add(gtkgui_drawMediaProgress,
+			       local_td->instance);
+		    g_idle_add(gtkgui_refreshbuttonstate,
+			       local_td->instance);
 		}
 #endif
 	    }
@@ -1027,13 +1029,15 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 	    if (local_td->instance->mediaPercent == 0) {
 		if ((int) local_td->instance->mediaLength > 1)
 		    lastmediapercent = local_td->instance->mediaPercent;
-		    local_td->instance->mediaPercent =
-			(int) ((local_td->instance->mediaPos * 100) /
-			       local_td->instance->mediaLength);
+		local_td->instance->mediaPercent =
+		    (int) ((local_td->instance->mediaPos * 100) /
+			   local_td->instance->mediaLength);
 #ifdef GTK_ENABLED
 		if (lastmediapercent != local_td->instance->mediaPercent) {
-		    g_idle_add(gtkgui_drawMediaProgress, local_td->instance);
-		    g_idle_add(gtkgui_refreshbuttonstate, local_td->instance);
+		    g_idle_add(gtkgui_drawMediaProgress,
+			       local_td->instance);
+		    g_idle_add(gtkgui_refreshbuttonstate,
+			       local_td->instance);
 		}
 #endif
 	    }
@@ -1053,7 +1057,7 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 		    printf("mediaTime = %f\n",
 			   local_td->instance->mediaTime);
 #ifdef GTK_ENABLED
-//		g_idle_add(gtkgui_draw, local_td->instance);
+//              g_idle_add(gtkgui_draw, local_td->instance);
 #endif
 	    } else {
 		msg = strstr(buffer, "\rV:");
@@ -1121,7 +1125,7 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 	    pthread_mutex_lock(&(local_td->instance->control_mutex));
 	    local_td->instance->js_state = JS_STATE_WAITING;
 	    pthread_mutex_unlock(&(local_td->instance->control_mutex));
-	
+
 	}
 
 	pthread_testcancel();
@@ -1161,18 +1165,20 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 		*maybeplaylist = 1;
 	    } else {
 		result->tryagain = TRYAGAIN_FALSE;
-	    	result->errorcode = ERROR_NOT_PLAYLIST;
+		result->errorcode = ERROR_NOT_PLAYLIST;
 	    }
 	    break;
 	}
 
+ /*
 	pthread_testcancel();
 	if (strstr(buffer, "File not found") != NULL) {
 	    result->tryagain = TRYAGAIN_FALSE;
 	    result->errorcode = ERROR_FILE_NOT_FOUND;
 	    break;
 	}
-
+*/
+    
 	pthread_testcancel();
 	if (strstr(buffer, "Error while decoding") != NULL) {
 	    if (!isMms(local_url, local_td->instance->nomediacache)) {
@@ -1240,9 +1246,13 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 		local_td->instance->state = STATE_DOWNLOADING;
 		if (local_list->retrieved == 1) {
 		    local_td->instance->state = STATE_PLAYING;
-		    pthread_mutex_lock(&(local_td->instance->control_mutex));
+		    pthread_mutex_lock(&
+				       (local_td->instance->
+					control_mutex));
 		    local_td->instance->js_state = JS_STATE_PLAYING;
-		    pthread_mutex_unlock(&(local_td->instance->control_mutex));
+		    pthread_mutex_unlock(&
+					 (local_td->instance->
+					  control_mutex));
 		    pthread_mutex_unlock(&
 					 (local_td->instance->
 					  playlist_mutex));
@@ -1269,6 +1279,16 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 	    break;
 	}
 
+/*	pthread_testcancel();
+	if (strstr(buffer, "Win32 LoadLibrary failed to load") != NULL) {
+	    if (local_td->instance->useragent != NULL) {
+		free(local_td->instance->useragent);
+		local_td->instance->useragent = NULL;
+		result->tryagain = TRYAGAIN_TRUE;
+		break;
+	    }
+	}
+*/
 	pthread_testcancel();
 	if (strstr(buffer, "Exiting") != NULL) {
 	    if (DEBUG)
@@ -1310,7 +1330,7 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 	}
 
 	pthread_testcancel();
-        if (strstr(buffer, "execv") != NULL) {
+	if (strstr(buffer, "execv") != NULL) {
 	    if (DEBUG)
 		printf
 		    ("----player thread: breaking read loop - execv failed\n");
@@ -1334,6 +1354,36 @@ PlayResult *playNode(ThreadData * local_td, Node * local_list,
 	    break;
 	}
 
+	pthread_testcancel();
+	if (strstr(buffer, "decoder is not supposed") != NULL) {
+	    if (DEBUG)
+		printf
+		    ("----player thread: breaking read loop - codec failure\n");
+#ifdef GTK_ENABLED
+	    snprintf(local_td->instance->lastmessage, 1024, "Error: %s %s",
+		     local_td->argv[0], buffer);
+	    g_idle_add(gtkgui_message, local_td->instance);
+#endif
+	    result->tryagain = TRYAGAIN_FALSE;
+	    result->errorcode = ERROR_CODEC_FAILURE;
+	    break;
+	}
+
+	pthread_testcancel();
+	if (strstr(buffer, "Failed to decode frame") != NULL) {
+	    if (DEBUG)
+		printf
+		    ("----player thread: breaking read loop - codec failure\n");
+#ifdef GTK_ENABLED
+	    snprintf(local_td->instance->lastmessage, 1024, "Error: %s %s",
+		     local_td->argv[0], buffer);
+	    g_idle_add(gtkgui_message, local_td->instance);
+#endif
+	    result->tryagain = TRYAGAIN_FALSE;
+	    result->errorcode = ERROR_CODEC_FAILURE;
+	    break;
+	}
+
 	pthread_testcancel();
 /*	commmented out to make BBC site work    
 	if (strstr(buffer, "everything done") != NULL) {
@@ -1489,10 +1539,10 @@ void *playPlaylist(void *td)
 
     pthread_testcancel();
 #ifdef X_ENABLED
-    DrawUI(local_td->w, local_td->instance, "Loading Media...", 0, -1);
+    DrawUI(local_td->w, local_td->instance, _("Loading Media..."), 0, -1);
 #endif
 #ifdef GTK_ENABLED
-    snprintf(local_td->instance->lastmessage, 1024, "Loading Media...");
+    snprintf(local_td->instance->lastmessage, 1024, _("Loading Media..."));
     g_idle_add(gtkgui_message, local_td->instance);
 #endif
 
@@ -1661,6 +1711,9 @@ void *playPlaylist(void *td)
 	    local_url[1023] = '\0';
 	    local_mmsstream = local_list->mmsstream;
 	}
+	if (local_td->instance->rtsp_use_http
+	    && strncmp(local_url, "rtsp://", 7) == 0)
+	    memcpy(local_url, "http://", 7);
 
 	pthread_mutex_unlock(&(local_td->instance->playlist_mutex));
 	pthread_cleanup_pop(0);
@@ -1694,8 +1747,7 @@ void *playPlaylist(void *td)
 
 	    if (DEBUG)
 		printf("----player thread: playing url: %s\n", local_url);
-	    strlcpy(message, "Playing ", 1024);
-	    strlcat(message, local_list->url, 1000);
+	    snprintf(message, 1024, _("Playing %s"), local_list->url);
 #ifdef X_ENABLED
 	    DrawUI(local_td->w, local_td->instance, message, 0, -1);
 #endif
@@ -1721,7 +1773,17 @@ void *playPlaylist(void *td)
 	    }
 
 	    if (local_td->instance->rtsp_use_tcp == 1) {
-		snprintf(buffer, 1024, "-rtsp-stream-over-tcp");
+	    	if (strncmp(local_list->url, "rtsp", 4) == 0) {
+		    snprintf(buffer, 1024, "-rtsp-stream-over-tcp");
+		    local_td->argv[argc++] = strdup(buffer);
+		}
+	    }
+
+	    if (local_td->instance->useragent) {
+		snprintf(buffer, 1024, "-user-agent");
+		local_td->argv[argc++] = strdup(buffer);
+		snprintf(buffer, 1024, "%s",
+			 local_td->instance->useragent);
 		local_td->argv[argc++] = strdup(buffer);
 	    }
 
@@ -1751,6 +1813,7 @@ void *playPlaylist(void *td)
 			 local_td->instance->cachesize);
 		local_td->argv[argc++] = strdup(message);
 
+	    	mmsToHttp(local_url,local_list->url);
 		local_td->argv[argc++] = strdup(local_url);
 		if (local_td->instance->keep_download) {
 		    snprintf(mmsplaylist, sizeof(mmsplaylist),
@@ -1821,7 +1884,7 @@ void *playPlaylist(void *td)
 		local_td->instance->js_state = JS_STATE_TRANSITIONING;
 		pthread_mutex_unlock(&(local_td->instance->control_mutex));
 
- 	        if (DEBUG)
+		if (DEBUG)
 		    printf
 			("----player thread: playNode returned = %d\n",
 			 result->retval);
@@ -1833,10 +1896,18 @@ void *playPlaylist(void *td)
 #ifdef GTK_ENABLED
 		g_idle_add(gtkgui_drawMediaProgress, local_td->instance);
 #endif
-
+		if (local_td->instance->targetplayer == 0) {
+		    local_td->instance->movie_height = 1;
+		    local_td->instance->movie_width = 1;
+#ifdef GTK_ENABLED
+		    g_idle_add(gtkgui_resize, local_td->instance);
+		    g_idle_add(gtkgui_draw, local_td->instance);
+#endif
+		}
+		
 		// this cancels the download of this media
 		// if there are more than 1, saves bandwidth
-		// also revents us from playing it again
+		// also prevents us from playing it again
 		pthread_mutex_lock(&(local_td->instance->playlist_mutex));
 		if (!(result->retval)) {
 		    local_list->played = 1;
@@ -1848,7 +1919,6 @@ void *playPlaylist(void *td)
 
 		//close the pipes to mplayer
 
-
 		assert(local_td->instance->control > 0);
 		close(local_td->instance->control);
 		local_td->instance->control = -1;
@@ -1890,7 +1960,7 @@ void *playPlaylist(void *td)
 			   local_td->instance->
 			   mediaCompleteWithErrorCallback);
 	    }
-
+	
 	} while (result->retval);
 
 	local_td->instance->currentnode = NULL;
@@ -1958,7 +2028,7 @@ void *playPlaylist(void *td)
 	local_td->argv[argc++] = NULL;
     }
     pthread_mutex_unlock(&(local_td->instance->control_mutex));
-
+    pthread_mutex_unlock(&(local_td->instance->playlist_cond_mutex));
     // since thread is dead, reset thread state variables.
     local_td->instance->threadsetup = 0;
     local_td->instance->threadlaunched = 0;
