@@ -48,8 +48,15 @@
 #include "../pixmaps/logo.xpm"
 #endif
 
+#ifdef HAVE_GETTEXT
+#ifdef X_ENABLED
+#include <locale.h>
+#endif
+#endif
+
 extern int errno;
 extern int DEBUG;
+extern int instance_counter;
 
 int32 STREAMBUFSIZE = 0X0FFFFFFF;
 
@@ -118,7 +125,7 @@ nsPluginInstanceBase *NS_NewPluginInstance(nsPluginCreateData *
 	new nsPluginInstance(aCreateDataStruct->instance);
 
     New(plugin, aCreateDataStruct);
-    return plugin;
+    return plugin;    
 }
 
 void NS_DestroyPluginInstance(nsPluginInstanceBase * aPlugin)
@@ -137,6 +144,8 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
 {
     int i;
 
+    instance_counter++;
+
     toolkitok = toolkitOk(mInstance, &moz_toolkit, &plug_toolkit);
 
     mScriptablePeer = getScriptablePeer();
@@ -270,6 +279,7 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     af = NULL;
     loop = -1;
     rtsp_use_tcp = 0;
+    rtsp_use_http = 0;
     keep_download = 0;
     maintain_aspect = 1;
     download_dir = strdup(getenv("HOME"));
@@ -291,6 +301,8 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
     enable_mpeg = 1;
     enable_mp3 = 1;
     enable_ogg = 1;
+    enable_midi = 0;
+    enable_pls = 1;
 
     // JavaScript
     paused = 0;
@@ -306,6 +318,13 @@ mInitialized(FALSE), mScriptablePeer(NULL), mControlsScriptablePeer(NULL)
 
     LoadConfigFile(this);
 
+#ifdef HAVE_GETTEXT
+#ifdef X_ENABLED
+    setlocale(LC_ALL, "");
+#endif
+    textdomain("mplayerplug-in");
+    bind_textdomain_codeset("mplayerplug-in", "utf-8");
+#endif
 }
 
 nsPluginInstance::~nsPluginInstance()
@@ -320,6 +339,7 @@ nsPluginInstance::~nsPluginInstance()
     gdk_flush();
 #endif
 
+    instance_counter--;
 
     mInstance = NULL;
     mInitialized = FALSE;
@@ -374,9 +394,11 @@ void nsPluginInstance::shutdown()
 		("Thread is setup but waiting for signal so we need to shut it down\n");
 	signalPlayerThread(this);
 	threadsignaled = 1;
+
     }
 
     if (threadsetup == 1) {
+
 	if (threadlaunched == 1) {
 	    pthread_mutex_lock(&control_mutex);
 	    if (paused == 1) {
@@ -384,17 +406,20 @@ void nsPluginInstance::shutdown()
 		paused = 0;
 	    }
 	    sendCommand(this, "quit\n");
-    	    pthread_mutex_lock(&read_mutex);
+#ifndef BSD
+	    pthread_mutex_lock(&read_mutex);
+#endif
 	    cancelled = 1;
-    	    pthread_mutex_unlock(&read_mutex);
+#ifndef BSD
+	    pthread_mutex_unlock(&read_mutex);
+#endif
 	    pthread_mutex_unlock(&control_mutex);
 	    pthread_cancel(player_thread);
 	    pthread_join(player_thread, NULL);
 	    js_state = JS_STATE_UNDEFINED;
-	    
+
 	}
     }
-
 #ifdef GTK_ENABLED
     while (g_idle_remove_by_data(this)) {
 	if (DEBUG)
@@ -403,7 +428,7 @@ void nsPluginInstance::shutdown()
 #endif
 
     if (pid != 0) {
-        killmplayer(this);
+	killmplayer(this);
     }
 
     if (DEBUG)
@@ -431,11 +456,15 @@ void nsPluginInstance::shutdown()
 	if (targetplayer != 0) {
 	    g_signal_handler_disconnect(GTK_OBJECT(gtkwidget),
 					delete_signal_id);
+	    g_signal_handler_disconnect(GTK_OBJECT(gtkwidget),
+					visible_signal_id);
 	    if (GTK_IS_WIDGET(button_window))
 		gtk_widget_destroy(button_window);
+
 	}
 	if (GTK_IS_WIDGET(gtkwidget))
 	    gtk_widget_destroy(gtkwidget);
+
     }
 #endif
 
@@ -651,7 +680,6 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	return NPERR_NO_ERROR;
     }
 
-
     if ((Window) window != (Window) aWindow->window) {
 	if (DEBUG)
 	    printf("New window! old: 0x%li    new 0x%li\n",
@@ -709,6 +737,7 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	    gtkwidget = gtk_window_new(GTK_WINDOW_POPUP);
 	} else {
 	    gtkwidget = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+	    button_window = gtk_window_new(GTK_WINDOW_POPUP);
 #ifdef GTK2_ENABLED
 	    delete_signal_id =
 		g_signal_connect(GTK_OBJECT(gtkwidget), "delete_event",
@@ -725,6 +754,7 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 				 "mplayerplug-in player");
 
 	}
+
 	gtk_widget_realize(gtkwidget);
 	gdk_flush();
 	gtk_widget_add_events(gtkwidget, GDK_BUTTON_PRESS_MASK);
@@ -736,6 +766,8 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	if (targetplayer == 1) {
 	    window_width = 400;
 	    window_height = 200;
+	    movie_width = 400;
+	    movie_height = 200;
 	}
 	if (DEBUG > 1)
 	    printf("setting window size\n");
@@ -749,13 +781,6 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 	gtk_widget_set_usize(gtkwidget, window_width, window_height);
 #endif
 
-
-
-#ifdef HAVE_GETTEXT
-	textdomain("mplayerplug-in");
-	bind_textdomain_codeset("mplayerplug-in", "utf-8");
-#endif
-
 	popup_menu = GTK_MENU(gtk_menu_new());
 	menuitem_play =
 	    GTK_MENU_ITEM(gtk_menu_item_new_with_label(_("Play")));
@@ -889,12 +914,22 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 				 "button_press_event",
 				 G_CALLBACK(popup_handler),
 				 GTK_OBJECT(popup_menu));
+	gtk_widget_set_size_request(GTK_WIDGET(fixed_container),
+				    window_width, window_height);
+	gtk_widget_set_size_request(GTK_WIDGET(fixed_event_box),
+				    window_width, window_height);
+
 #endif
 #ifdef GTK1_ENABLED
 	gtk_signal_connect_object(GTK_OBJECT(fixed_event_box),
 				  "button_press_event",
 				  GTK_SIGNAL_FUNC(popup_handler),
 				  GTK_OBJECT(popup_menu));
+	gtk_widget_set_usize(GTK_WIDGET(fixed_container), window_width,
+			     window_height);
+	gtk_widget_set_usize(GTK_WIDGET(fixed_event_box), window_width,
+			     window_height);
+
 #endif
 	gtk_container_add(GTK_CONTAINER(fixed_event_box), fixed_container);
 
@@ -973,6 +1008,17 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 
 	gtk_misc_set_alignment(GTK_MISC(status), 0, 0);
 	gtk_label_set_line_wrap(status, TRUE);
+	drawing_area = gtk_socket_new();
+	gtk_widget_set_events(drawing_area, GDK_BUTTON_PRESS_MASK);
+	if (mode == NP_EMBED) {
+	    gtk_fixed_put(GTK_FIXED(fixed_container), drawing_area, 0, 0);
+	} else {
+	    gtk_fixed_put(GTK_FIXED(fixed_container),
+			  drawing_area, 10, 100);
+	}
+	gtk_widget_show(drawing_area);
+
+
 	gtk_container_add(GTK_CONTAINER(gtkwidget), fixed_event_box);
 	if (!hidestatus)
 	    gtk_widget_show(GTK_WIDGET(status));
@@ -991,6 +1037,8 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 #endif
 
 #ifdef GTK2_ENABLED
+	    gtk_widget_modify_bg(gtkwidget, GTK_STATE_NORMAL,
+				 &black);
 	    gtk_widget_modify_bg(fixed_event_box, GTK_STATE_NORMAL,
 				 &black);
 	    gtk_widget_modify_fg(GTK_WIDGET(status), GTK_STATE_NORMAL,
@@ -1004,9 +1052,6 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 
 	gdk_flush();
 	InitPixbufs(this);
-
-	if (DEBUG > 1)
-	    printf("all setup and ready to reparent and map\n");
 #ifdef GTK_ENABLED
 
 	if (targetplayer == 0) {
@@ -1014,9 +1059,14 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 			    GDK_WINDOW_XWINDOW(gtkwidget->window), window,
 			    0, 0);
 	    gtk_widget_map(gtkwidget);
+	} else {
+	    gtk_widget_show(button_window);
+	    XReparentWindow(GDK_WINDOW_XDISPLAY(button_window->window),
+			    GDK_WINDOW_XWINDOW(button_window->window),
+			    window, 0, 0);
+	    gtk_widget_map(button_window);
 	}
 #endif
-
 	if (panel_drawn == 0 && autostart == 0 && nomediacache == 1) {
 	    if (showcontrols) {
 		panel_height = window_height;
@@ -1025,10 +1075,6 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 #endif
 	    }
 	}
-
-	if (DEBUG > 1)
-	    printf("reparent and map done\n");
-
 #endif
 
     } else {
@@ -1072,7 +1118,7 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 			    pthread_mutex_lock(&control_mutex);
 			    js_state = JS_STATE_READY;
 			    pthread_mutex_unlock(&control_mutex);
-			}			
+			}
 		    }
 		}
 
@@ -1090,7 +1136,7 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 			    pthread_mutex_lock(&control_mutex);
 			    js_state = JS_STATE_READY;
 			    pthread_mutex_unlock(&control_mutex);
-			}			
+			}
 		    }
 		}
 	    }
@@ -1117,23 +1163,75 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 #ifdef GTK2_ENABLED
 	gtk_widget_set_size_request(GTK_WIDGET(status),
 				    window_width - 20, window_height - 25);
+	gtk_widget_set_size_request(GTK_WIDGET(gtkwidget),
+				    window_width, window_height);
+
 #endif
 #ifdef GTK1_ENABLED
 	gtk_widget_set_usize(GTK_WIDGET(status),
 			     window_width - 20, window_height - 25);
+	gtk_widget_set_usize(GTK_WIDGET(gtkwidget),
+			     window_width, window_height);
 #endif
+	if (panel_drawn == 1) {
+	    height = 16;
+	    width = 21;
+
+	    multiplier = 0;
+
+	    if (mmsstream == 0) {
+		gtk_fixed_move(GTK_FIXED(fixed_container),
+			       GTK_WIDGET(rew_event_box),
+			       width * multiplier++,
+			       window_height - height);
+	    }
+	    gtk_fixed_move(GTK_FIXED(fixed_container),
+			   GTK_WIDGET(play_event_box),
+			   width * multiplier++, window_height - height);
+	    gtk_fixed_move(GTK_FIXED(fixed_container),
+			   GTK_WIDGET(pause_event_box),
+			   width * multiplier++, window_height - height);
+	    gtk_fixed_move(GTK_FIXED(fixed_container),
+			   GTK_WIDGET(stop_event_box),
+			   width * multiplier++, window_height - height);
+	    if (mmsstream == 0) {
+		gtk_fixed_move(GTK_FIXED(fixed_container),
+			       GTK_WIDGET(ff_event_box),
+			       width * multiplier++,
+			       window_height - height);
+	    }
+	    if (GTK_IS_WIDGET(mediaprogress_bar)) {
+		gtk_fixed_move(GTK_FIXED(fixed_container),
+			       GTK_WIDGET(mediaprogress_bar),
+			       (width * multiplier + 10),
+			       window_height - height + 2);
+
+		gtk_widget_set_usize(GTK_WIDGET(mediaprogress_bar),
+				     window_width -
+				     (width * (multiplier + 1) + 20),
+				     height - 4);
+	    }
+	    gtk_fixed_move(GTK_FIXED(fixed_container),
+			   GTK_WIDGET(fs_event_box),
+			   (window_width - width), window_height - height);
+
+	}
+	if (progress_bar != NULL) {
+	    gtk_fixed_move(GTK_FIXED(fixed_container),
+			   GTK_WIDGET(progress_bar),
+			   10, window_height - 50);
+
+	    gtk_widget_set_usize(GTK_WIDGET(progress_bar),
+				 window_width - 20, 15);
+	}
+
     }
     if (gtkwidget == NULL)
 	return NPERR_NO_ERROR;
 
     if (targetplayer == 0)
 	gtk_widget_show(gtkwidget);
-/*
-    XResizeWindow(GDK_WINDOW_XDISPLAY(gtkwidget->window),
-		  GDK_WINDOW_XWINDOW(gtkwidget->window),
-		  aWindow->width, aWindow->height);
-    XResizeWindow(display, window, aWindow->width, aWindow->height);
-*/
+
     if (DEBUG > 1)
 	printf("resize is complete\n");
 
@@ -1145,52 +1243,6 @@ NPError nsPluginInstance::SetWindow(NPWindow * aWindow)
 #endif
 	}
     }
-
-    if (panel_drawn == 1 && mode == NP_FULL) {
-	if (panel_height >= 32) {
-	    height = 32;
-	    width = 43;
-	} else {
-	    height = 16;
-	    width = 21;
-	}
-
-	multiplier = 0;
-
-	if (mmsstream == 0) {
-	    gtk_fixed_move(GTK_FIXED(fixed_container),
-			   GTK_WIDGET(rew_event_box), width * multiplier++,
-			   aWindow->height - height);
-	}
-	gtk_fixed_move(GTK_FIXED(fixed_container),
-		       GTK_WIDGET(play_event_box), width * multiplier++,
-		       aWindow->height - height);
-	gtk_fixed_move(GTK_FIXED(fixed_container),
-		       GTK_WIDGET(pause_event_box), width * multiplier++,
-		       aWindow->height - height);
-	gtk_fixed_move(GTK_FIXED(fixed_container),
-		       GTK_WIDGET(stop_event_box), width * multiplier++,
-		       aWindow->height - height);
-	if (mmsstream == 0) {
-	    gtk_fixed_move(GTK_FIXED(fixed_container),
-			   GTK_WIDGET(ff_event_box), width * multiplier++,
-			   aWindow->height - height);
-	}
-	if (mediaprogress_bar != NULL) {
-	    gtk_fixed_move(GTK_FIXED(fixed_container),
-			   GTK_WIDGET(mediaprogress_bar),
-			   (width * multiplier + 10),
-			   aWindow->height - height + 2);
-	    gtk_widget_set_usize(GTK_WIDGET(mediaprogress_bar),
-				 aWindow->width -
-				 (width * (multiplier + 1) + 20),
-				 height - 4);
-	}
-	gtk_fixed_move(GTK_FIXED(fixed_container),
-		       GTK_WIDGET(fs_event_box),
-		       (aWindow->width - width), aWindow->height - height);
-
-    }
 #endif
 
     if (DEBUG >= 2) {
@@ -1279,9 +1331,12 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 		if (n->fname != NULL)
 		    remove(n->fname);
 	    }
-	    printf
-		("checking to see if we need to make a button\nn->url=%s\nurl=%s\nhref=%s\n",
-		 n->url, url, href);
+
+	    if (DEBUG)
+		printf
+		    ("checking to see if we need to make a button\nn->url=%s\nurl=%s\nhref=%s\n",
+		     n->url, url, (href == NULL) ? "(NULL)" : href);
+
 	    if (n->url != NULL && url != NULL) {
 		if ((strncmp(n->url, url, 1024) == 0) && href != NULL) {
 		    if (srcToButton(n->fname, this)) {
@@ -1296,6 +1351,21 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 			NPN_GetURL(mInstance, href, NULL);
 			return NPERR_NO_ERROR;
 		    }
+		} else if ( strstr(n->url, url) && href != NULL) {
+		    // we are in this block because the url is a relative url and so
+		    // it is a subset of the fully qualified n->url
+		    if (srcToButton(n->fname, this)) {
+			n->play = 0;
+			pthread_mutex_unlock(&playlist_mutex);
+			return NPERR_NO_ERROR;
+		    } else {
+			n = newNode();
+			snprintf(n->url, 1024, "%s", href);
+			addToEnd(td->list, n);
+			pthread_mutex_unlock(&playlist_mutex);
+			NPN_GetURL(mInstance, href, NULL);
+			return NPERR_NO_ERROR;
+		    }
 		}
 	    }
 
@@ -1306,12 +1376,13 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 		buildPlaylist(this, n->fname, n);
 		if (mode == NP_FULL || noembed == 1) {
 #ifdef X_ENABLED
-		    DrawUI(widget, this, "Download Complete", 0, 99);
+		    DrawUI(widget, this, _("Download Complete"), 0, 99);
 #endif
 #ifdef GTK_ENABLED
 		    if (status != NULL) {
-			gtk_label_set_text(status, "Download Complete");
+			gtk_label_set_text(status, _("Download Complete"));
 		    }
+		    g_idle_add(gtkgui_save_enable, this);
 		    if (progress_bar != NULL) {
 			gtk_progress_bar_update(progress_bar, 1.0);
 			if (n->next == NULL) {
@@ -1354,7 +1425,7 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 				    pthread_mutex_lock(&control_mutex);
 				    js_state = JS_STATE_READY;
 				    pthread_mutex_unlock(&control_mutex);
-				
+
 				}
 			    }
 
@@ -1435,7 +1506,7 @@ NPError nsPluginInstance::DestroyStream(NPStream * stream, NPError reason)
 			    pthread_mutex_lock(&control_mutex);
 			    js_state = JS_STATE_READY;
 			    pthread_mutex_unlock(&control_mutex);
-			}			
+			}
 		    }
 		}
 	    }
@@ -1524,7 +1595,7 @@ void nsPluginInstance::URLNotify(const char *url, NPReason reason,
 				pthread_mutex_lock(&control_mutex);
 				js_state = JS_STATE_READY;
 				pthread_mutex_unlock(&control_mutex);
-			    
+
 			    }
 			}
 		    }
@@ -1541,7 +1612,7 @@ int32 nsPluginInstance::WriteReady(NPStream * stream)
 {
     Node *n;
 
-    if (state == STATE_PLAY_CANCELLED)
+    if (state == STATE_PLAY_CANCELLED || cancelled == 1)
 	return -1;
 
     if (td == NULL)
@@ -1607,7 +1678,7 @@ int32 nsPluginInstance::WriteReady(NPStream * stream)
 			pthread_mutex_lock(&control_mutex);
 			js_state = JS_STATE_READY;
 			pthread_mutex_unlock(&control_mutex);
-		    
+
 		    }
 		}
 	    }
@@ -1616,6 +1687,15 @@ int32 nsPluginInstance::WriteReady(NPStream * stream)
 	    if (strlen(n->fname) == 0) {
 		snprintf(n->fname, 1024, "%s",
 			 tempnam("/tmp", "mplayerplug-inXXXXXX"));
+		if (strstr(mimetype, "midi") != NULL) {
+		    strlcat(n->fname, ".mid", 1024);
+		}
+		if (strstr(mimetype, "mp3") != NULL) {
+		    strlcat(n->fname, ".mp3", 1024);
+		}
+		if (strstr(mimetype, "audio/mpeg") != NULL) {
+		    strlcat(n->fname, ".mp3", 1024);
+		}
 		if (DEBUG)
 		    printf("WR tempname: %s\n", n->fname);
 
@@ -1683,8 +1763,9 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
     char message[1024];
     char *burl;
 
-    if (state == STATE_PLAY_CANCELLED)
+    if (state == STATE_PLAY_CANCELLED || cancelled == 1)
 	return -1;
+
     if (td == NULL)
 	return -1;
 
@@ -1877,6 +1958,9 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 		}
 	    }
 	    if (n->play == 1) {
+		if (n->localcache != NULL && nomediacache == 1) {
+		    fclose(n->localcache);
+		}
 		if (n->mmsstream == 1) {
 		    if (threadsignaled == 0) {
 			if (autostart) {
@@ -1908,7 +1992,7 @@ int32 nsPluginInstance::Write(NPStream * stream, int32 offset, int32 len,
 				    pthread_mutex_lock(&control_mutex);
 				    js_state = JS_STATE_READY;
 				    pthread_mutex_unlock(&control_mutex);
-				
+
 				}
 			    }
 			}
@@ -1992,7 +2076,6 @@ void nsPluginInstance::Play()
 	signalPlayerThread(this);
 	threadsignaled = 1;
     }
-
     // normal state, the media is paused or stopped and the thread is active.
     if (paused == 1) {
 	if (DEBUG)
@@ -2362,7 +2445,7 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 
 #ifdef GTK_ENABLED
     int multiplier, height = 0, width;
-    int x, y, disp_x, pos_x, disp_y, pos_y;
+    int x, y, disp_x, pos_x, disp_y, pos_y, button_top;
     GdkColormap *colormap;
     GdkColor black;
 #endif
@@ -2510,37 +2593,40 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 		height = 16;
 		width = 21;
 
+		if (targetplayer == 0) {
+		    button_top = win_height - height;
+		} else {
+		    button_top = win_height;
+		}
+
 		multiplier = 0;
 
 		if (mmsstream == 0 && win_width > 126) {
 		    gtk_fixed_move(GTK_FIXED(fixed_container),
 				   GTK_WIDGET(rew_event_box),
-				   width * multiplier++,
-				   win_height - height);
+				   width * multiplier++, button_top);
 		}
 		gtk_fixed_move(GTK_FIXED(fixed_container),
 			       GTK_WIDGET(play_event_box),
-			       width * multiplier++, win_height - height);
+			       width * multiplier++, button_top);
 		if (win_width > 126)
 		    gtk_fixed_move(GTK_FIXED(fixed_container),
 				   GTK_WIDGET(pause_event_box),
-				   width * multiplier++,
-				   win_height - height);
+				   width * multiplier++, button_top);
 		gtk_fixed_move(GTK_FIXED(fixed_container),
 			       GTK_WIDGET(stop_event_box),
-			       width * multiplier++, win_height - height);
+			       width * multiplier++, button_top);
 		if (mmsstream == 0 && win_width > 126) {
 		    gtk_fixed_move(GTK_FIXED(fixed_container),
 				   GTK_WIDGET(ff_event_box),
-				   width * multiplier++,
-				   win_height - height);
+				   width * multiplier++, button_top);
 		}
 
 		if (mediaprogress_bar != NULL) {
 		    gtk_fixed_move(GTK_FIXED(fixed_container),
 				   GTK_WIDGET(mediaprogress_bar),
 				   (width * multiplier + 10),
-				   win_height - height + 2);
+				   button_top + 2);
 		    gtk_widget_set_usize(GTK_WIDGET(mediaprogress_bar),
 					 win_width -
 					 (width * (multiplier + 1) + 20),
@@ -2549,8 +2635,7 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 		if (win_width > 126)
 		    gtk_fixed_move(GTK_FIXED(fixed_container),
 				   GTK_WIDGET(fs_event_box),
-				   (win_width - width),
-				   win_height - height);
+				   (win_width - width), button_top);
 
 
 	    }
@@ -2679,6 +2764,7 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 		gtk_window_fullscreen(GTK_WINDOW(gtkwidget));
 	    }
 #endif
+
 #ifdef GTK1_ENABLED
 	    fs_window = gtk_window_new(GTK_WINDOW_POPUP);
 	    gtk_widget_add_events(fs_window, GDK_BUTTON_PRESS_MASK);
@@ -2717,6 +2803,7 @@ void nsPluginInstance::SetFullscreen(PRBool value)
 		height = 16;
 		width = 21;
 
+
 		multiplier = 0;
 
 		if (mmsstream == 0) {
