@@ -204,6 +204,7 @@ class nsPluginInstance:public nsPluginInstanceBase {
     
 #ifdef GTK_ENABLED
     gulong delete_signal_id;
+    gulong visible_signal_id;
     GtkWidget *gtkwidget;
     GtkWidget *button_window;
     GtkWidget *gtkplug;
@@ -256,6 +257,7 @@ class nsPluginInstance:public nsPluginInstanceBase {
     GtkWidget *conf_enable_helix;
     GtkWidget *conf_nomediacache;
     GtkWidget *conf_rtsptcp;
+    GtkWidget *conf_rtsphttp;
     GtkWidget *conf_enable_wmp;
     GtkWidget *conf_enable_qt;
     GtkWidget *conf_enable_rm;
@@ -263,7 +265,9 @@ class nsPluginInstance:public nsPluginInstanceBase {
     GtkWidget *conf_enable_ogg;
     GtkWidget *conf_enable_mpeg;
     GtkWidget *conf_enable_mp3;
-	
+    GtkWidget *conf_enable_midi;
+    GtkWidget *conf_enable_pls;
+    GtkWidget *conf_pauseonhide;	
 #endif
 
     pthread_t player_thread;
@@ -320,7 +324,7 @@ class nsPluginInstance:public nsPluginInstanceBase {
 #endif
 
 #ifdef X_ENABLED
-    XFontStruct *font;
+    XFontSet font;
     Pixmap logo;
     Pixmap logomask;
     Pixmap progress_left;
@@ -343,6 +347,7 @@ class nsPluginInstance:public nsPluginInstanceBase {
     char *af;
     int loop;
     int rtsp_use_tcp;
+    int rtsp_use_http;
     int keep_download;
     int maintain_aspect;
     int qt_speed;
@@ -367,6 +372,8 @@ class nsPluginInstance:public nsPluginInstanceBase {
     int enable_mpeg;
     int enable_mp3;
     int enable_ogg;
+    int enable_midi;
+    int enable_pls;
 
     // JavaScript State
     int paused;
